@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022 Google LLC
+# Copyright 2024 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 import warnings
 
+from google.api_core import exceptions as core_exceptions
 from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
+from google.api_core import retry_async as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.cloud.location import locations_pb2  # type: ignore
@@ -71,7 +73,6 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
                 the credentials from the environment.
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
-                This argument is ignored if ``channel`` is provided.
             scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                 service. These are only used when credentials are not specified and
                 are passed to :func:`google.auth.default`.
@@ -101,7 +102,7 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
         credentials: Optional[ga_credentials.Credentials] = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
-        channel: Optional[aio.Channel] = None,
+        channel: Optional[Union[aio.Channel, Callable[..., aio.Channel]]] = None,
         api_mtls_endpoint: Optional[str] = None,
         client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
         ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
@@ -115,21 +116,24 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
 
         Args:
             host (Optional[str]):
-                 The hostname to connect to.
+                 The hostname to connect to (default: 'dataplex.googleapis.com').
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
-                This argument is ignored if ``channel`` is provided.
+                This argument is ignored if a ``channel`` instance is provided.
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
-                This argument is ignored if ``channel`` is provided.
+                This argument is ignored if a ``channel`` instance is provided.
             scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                 service. These are only used when credentials are not specified and
                 are passed to :func:`google.auth.default`.
-            channel (Optional[aio.Channel]): A ``Channel`` instance through
-                which to make calls.
+            channel (Optional[Union[aio.Channel, Callable[..., aio.Channel]]]):
+                A ``Channel`` instance through which to make calls, or a Callable
+                that constructs and returns one. If set to None, ``self.create_channel``
+                is used to create the channel. If a Callable is given, it will be called
+                with the same arguments as used in ``self.create_channel``.
             api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                 If provided, it overrides the ``host`` argument and tries to create
                 a mutual TLS channel with client SSL credentials from
@@ -139,11 +143,11 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
                 private key bytes, both in PEM format. It is ignored if
                 ``api_mtls_endpoint`` is None.
             ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
-                for the grpc channel. It is ignored if ``channel`` is provided.
+                for the grpc channel. It is ignored if a ``channel`` instance is provided.
             client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                 A callback to provide client certificate bytes and private key bytes,
                 both in PEM format. It is used to configure a mutual TLS channel. It is
-                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
+                ignored if a ``channel`` instance or ``ssl_channel_credentials`` is provided.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
             client_info (google.api_core.gapic_v1.client_info.ClientInfo):
@@ -170,7 +174,7 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
         if client_cert_source:
             warnings.warn("client_cert_source is deprecated", DeprecationWarning)
 
-        if channel:
+        if isinstance(channel, aio.Channel):
             # Ignore credentials if a channel was passed.
             credentials = False
             # If a channel was explicitly provided, set it.
@@ -210,7 +214,9 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
         )
 
         if not self._grpc_channel:
-            self._grpc_channel = type(self).create_channel(
+            # initialize with the provided callable or the default channel
+            channel_init = channel or type(self).create_channel
+            self._grpc_channel = channel_init(
                 self._host,
                 # use the credentials which are saved
                 credentials=self._credentials,
@@ -476,6 +482,86 @@ class DataScanServiceGrpcAsyncIOTransport(DataScanServiceTransport):
             )
         return self._stubs["list_data_scan_jobs"]
 
+    @property
+    def generate_data_quality_rules(
+        self,
+    ) -> Callable[
+        [datascans.GenerateDataQualityRulesRequest],
+        Awaitable[datascans.GenerateDataQualityRulesResponse],
+    ]:
+        r"""Return a callable for the generate data quality rules method over gRPC.
+
+        Generates recommended DataQualityRule from a data
+        profiling DataScan.
+
+        Returns:
+            Callable[[~.GenerateDataQualityRulesRequest],
+                    Awaitable[~.GenerateDataQualityRulesResponse]]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "generate_data_quality_rules" not in self._stubs:
+            self._stubs["generate_data_quality_rules"] = self.grpc_channel.unary_unary(
+                "/google.cloud.dataplex.v1.DataScanService/GenerateDataQualityRules",
+                request_serializer=datascans.GenerateDataQualityRulesRequest.serialize,
+                response_deserializer=datascans.GenerateDataQualityRulesResponse.deserialize,
+            )
+        return self._stubs["generate_data_quality_rules"]
+
+    def _prep_wrapped_messages(self, client_info):
+        """Precompute the wrapped methods, overriding the base class method to use async wrappers."""
+        self._wrapped_methods = {
+            self.create_data_scan: gapic_v1.method_async.wrap_method(
+                self.create_data_scan,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.update_data_scan: gapic_v1.method_async.wrap_method(
+                self.update_data_scan,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.delete_data_scan: gapic_v1.method_async.wrap_method(
+                self.delete_data_scan,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.get_data_scan: gapic_v1.method_async.wrap_method(
+                self.get_data_scan,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.list_data_scans: gapic_v1.method_async.wrap_method(
+                self.list_data_scans,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.run_data_scan: gapic_v1.method_async.wrap_method(
+                self.run_data_scan,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.get_data_scan_job: gapic_v1.method_async.wrap_method(
+                self.get_data_scan_job,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.list_data_scan_jobs: gapic_v1.method_async.wrap_method(
+                self.list_data_scan_jobs,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.generate_data_quality_rules: gapic_v1.method_async.wrap_method(
+                self.generate_data_quality_rules,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+        }
+
     def close(self):
         return self.grpc_channel.close()
 
