@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022 Google LLC
+# Copyright 2024 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -61,7 +61,7 @@ class DataplexServiceTransport(abc.ABC):
 
         Args:
             host (Optional[str]):
-                 The hostname to connect to.
+                 The hostname to connect to (default: 'dataplex.googleapis.com').
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -124,6 +124,10 @@ class DataplexServiceTransport(abc.ABC):
             host += ":443"
         self._host = host
 
+    @property
+    def host(self):
+        return self._host
+
     def _prep_wrapped_messages(self, client_info):
         # Precompute the wrapped methods.
         self._wrapped_methods = {
@@ -355,6 +359,11 @@ class DataplexServiceTransport(abc.ABC):
                 default_timeout=60.0,
                 client_info=client_info,
             ),
+            self.run_task: gapic_v1.method.wrap_method(
+                self.run_task,
+                default_timeout=None,
+                client_info=client_info,
+            ),
             self.get_job: gapic_v1.method.wrap_method(
                 self.get_job,
                 default_retry=retries.Retry(
@@ -648,6 +657,15 @@ class DataplexServiceTransport(abc.ABC):
     ]:
         raise NotImplementedError()
 
+    @property
+    def run_task(
+        self,
+    ) -> Callable[
+        [service.RunTaskRequest],
+        Union[service.RunTaskResponse, Awaitable[service.RunTaskResponse]],
+    ]:
+        raise NotImplementedError()
+
     @property
     def get_job(
         self,
