@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022 Google LLC
+# Copyright 2024 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 import warnings
 
+from google.api_core import exceptions as core_exceptions
 from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
+from google.api_core import retry_async as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.cloud.location import locations_pb2  # type: ignore
@@ -74,7 +76,6 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
                 the credentials from the environment.
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
-                This argument is ignored if ``channel`` is provided.
             scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                 service. These are only used when credentials are not specified and
                 are passed to :func:`google.auth.default`.
@@ -104,7 +105,7 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
         credentials: Optional[ga_credentials.Credentials] = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
-        channel: Optional[aio.Channel] = None,
+        channel: Optional[Union[aio.Channel, Callable[..., aio.Channel]]] = None,
         api_mtls_endpoint: Optional[str] = None,
         client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
         ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
@@ -118,21 +119,24 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
 
         Args:
             host (Optional[str]):
-                 The hostname to connect to.
+                 The hostname to connect to (default: 'dataplex.googleapis.com').
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
-                This argument is ignored if ``channel`` is provided.
+                This argument is ignored if a ``channel`` instance is provided.
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
-                This argument is ignored if ``channel`` is provided.
+                This argument is ignored if a ``channel`` instance is provided.
             scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                 service. These are only used when credentials are not specified and
                 are passed to :func:`google.auth.default`.
-            channel (Optional[aio.Channel]): A ``Channel`` instance through
-                which to make calls.
+            channel (Optional[Union[aio.Channel, Callable[..., aio.Channel]]]):
+                A ``Channel`` instance through which to make calls, or a Callable
+                that constructs and returns one. If set to None, ``self.create_channel``
+                is used to create the channel. If a Callable is given, it will be called
+                with the same arguments as used in ``self.create_channel``.
             api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                 If provided, it overrides the ``host`` argument and tries to create
                 a mutual TLS channel with client SSL credentials from
@@ -142,11 +146,11 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
                 private key bytes, both in PEM format. It is ignored if
                 ``api_mtls_endpoint`` is None.
             ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
-                for the grpc channel. It is ignored if ``channel`` is provided.
+                for the grpc channel. It is ignored if a ``channel`` instance is provided.
             client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                 A callback to provide client certificate bytes and private key bytes,
                 both in PEM format. It is used to configure a mutual TLS channel. It is
-                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
+                ignored if a ``channel`` instance or ``ssl_channel_credentials`` is provided.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
             client_info (google.api_core.gapic_v1.client_info.ClientInfo):
@@ -173,7 +177,7 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
         if client_cert_source:
             warnings.warn("client_cert_source is deprecated", DeprecationWarning)
 
-        if channel:
+        if isinstance(channel, aio.Channel):
             # Ignore credentials if a channel was passed.
             credentials = False
             # If a channel was explicitly provided, set it.
@@ -213,7 +217,9 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
         )
 
         if not self._grpc_channel:
-            self._grpc_channel = type(self).create_channel(
+            # initialize with the provided callable or the default channel
+            channel_init = channel or type(self).create_channel
+            self._grpc_channel = channel_init(
                 self._host,
                 # use the credentials which are saved
                 credentials=self._credentials,
@@ -886,6 +892,32 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
             )
         return self._stubs["list_jobs"]
 
+    @property
+    def run_task(
+        self,
+    ) -> Callable[[service.RunTaskRequest], Awaitable[service.RunTaskResponse]]:
+        r"""Return a callable for the run task method over gRPC.
+
+        Run an on demand execution of a Task.
+
+        Returns:
+            Callable[[~.RunTaskRequest],
+                    Awaitable[~.RunTaskResponse]]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "run_task" not in self._stubs:
+            self._stubs["run_task"] = self.grpc_channel.unary_unary(
+                "/google.cloud.dataplex.v1.DataplexService/RunTask",
+                request_serializer=service.RunTaskRequest.serialize,
+                response_deserializer=service.RunTaskResponse.deserialize,
+            )
+        return self._stubs["run_task"]
+
     @property
     def get_job(self) -> Callable[[service.GetJobRequest], Awaitable[tasks.Job]]:
         r"""Return a callable for the get job method over gRPC.
@@ -1104,6 +1136,311 @@ class DataplexServiceGrpcAsyncIOTransport(DataplexServiceTransport):
             )
         return self._stubs["list_sessions"]
 
+    def _prep_wrapped_messages(self, client_info):
+        """Precompute the wrapped methods, overriding the base class method to use async wrappers."""
+        self._wrapped_methods = {
+            self.create_lake: gapic_v1.method_async.wrap_method(
+                self.create_lake,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.update_lake: gapic_v1.method_async.wrap_method(
+                self.update_lake,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.delete_lake: gapic_v1.method_async.wrap_method(
+                self.delete_lake,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_lakes: gapic_v1.method_async.wrap_method(
+                self.list_lakes,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.get_lake: gapic_v1.method_async.wrap_method(
+                self.get_lake,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_lake_actions: gapic_v1.method_async.wrap_method(
+                self.list_lake_actions,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.create_zone: gapic_v1.method_async.wrap_method(
+                self.create_zone,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.update_zone: gapic_v1.method_async.wrap_method(
+                self.update_zone,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.delete_zone: gapic_v1.method_async.wrap_method(
+                self.delete_zone,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_zones: gapic_v1.method_async.wrap_method(
+                self.list_zones,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.get_zone: gapic_v1.method_async.wrap_method(
+                self.get_zone,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_zone_actions: gapic_v1.method_async.wrap_method(
+                self.list_zone_actions,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.create_asset: gapic_v1.method_async.wrap_method(
+                self.create_asset,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.update_asset: gapic_v1.method_async.wrap_method(
+                self.update_asset,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.delete_asset: gapic_v1.method_async.wrap_method(
+                self.delete_asset,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_assets: gapic_v1.method_async.wrap_method(
+                self.list_assets,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.get_asset: gapic_v1.method_async.wrap_method(
+                self.get_asset,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_asset_actions: gapic_v1.method_async.wrap_method(
+                self.list_asset_actions,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.create_task: gapic_v1.method_async.wrap_method(
+                self.create_task,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.update_task: gapic_v1.method_async.wrap_method(
+                self.update_task,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.delete_task: gapic_v1.method_async.wrap_method(
+                self.delete_task,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_tasks: gapic_v1.method_async.wrap_method(
+                self.list_tasks,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.get_task: gapic_v1.method_async.wrap_method(
+                self.get_task,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_jobs: gapic_v1.method_async.wrap_method(
+                self.list_jobs,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.run_task: gapic_v1.method_async.wrap_method(
+                self.run_task,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+            self.get_job: gapic_v1.method_async.wrap_method(
+                self.get_job,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.cancel_job: gapic_v1.method_async.wrap_method(
+                self.cancel_job,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.create_environment: gapic_v1.method_async.wrap_method(
+                self.create_environment,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.update_environment: gapic_v1.method_async.wrap_method(
+                self.update_environment,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.delete_environment: gapic_v1.method_async.wrap_method(
+                self.delete_environment,
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_environments: gapic_v1.method_async.wrap_method(
+                self.list_environments,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.get_environment: gapic_v1.method_async.wrap_method(
+                self.get_environment,
+                default_retry=retries.AsyncRetry(
+                    initial=1.0,
+                    maximum=10.0,
+                    multiplier=1.3,
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ServiceUnavailable,
+                    ),
+                    deadline=60.0,
+                ),
+                default_timeout=60.0,
+                client_info=client_info,
+            ),
+            self.list_sessions: gapic_v1.method_async.wrap_method(
+                self.list_sessions,
+                default_timeout=None,
+                client_info=client_info,
+            ),
+        }
+
     def close(self):
         return self.grpc_channel.close()
 
