@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022 Google LLC
+# Copyright 2024 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,72 +31,169 @@ __protobuf__ = proto.module(
 
 
 class DataProfileSpec(proto.Message):
-    r"""DataProfileScan related setting."""
+    r"""DataProfileScan related setting.
+
+    Attributes:
+        sampling_percent (float):
+            Optional. The percentage of the records to be selected from
+            the dataset for DataScan.
+
+            -  Value can range between 0.0 and 100.0 with up to 3
+               significant decimal digits.
+            -  Sampling is not applied if ``sampling_percent`` is not
+               specified, 0 or
+
+            100.
+        row_filter (str):
+            Optional. A filter applied to all rows in a
+            single DataScan job. The filter needs to be a
+            valid SQL expression for a WHERE clause in
+            BigQuery standard SQL syntax.
+            Example: col1 >= 0 AND col2 < 10
+        post_scan_actions (google.cloud.dataplex_v1.types.DataProfileSpec.PostScanActions):
+            Optional. Actions to take upon job
+            completion..
+        include_fields (google.cloud.dataplex_v1.types.DataProfileSpec.SelectedFields):
+            Optional. The fields to include in data profile.
+
+            If not specified, all fields at the time of profile scan job
+            execution are included, except for ones listed in
+            ``exclude_fields``.
+        exclude_fields (google.cloud.dataplex_v1.types.DataProfileSpec.SelectedFields):
+            Optional. The fields to exclude from data profile.
+
+            If specified, the fields will be excluded from data profile,
+            regardless of ``include_fields`` value.
+    """
+
+    class PostScanActions(proto.Message):
+        r"""The configuration of post scan actions of DataProfileScan
+        job.
+
+        Attributes:
+            bigquery_export (google.cloud.dataplex_v1.types.DataProfileSpec.PostScanActions.BigQueryExport):
+                Optional. If set, results will be exported to
+                the provided BigQuery table.
+        """
+
+        class BigQueryExport(proto.Message):
+            r"""The configuration of BigQuery export post scan action.
+
+            Attributes:
+                results_table (str):
+                    Optional. The BigQuery table to export DataProfileScan
+                    results to. Format:
+                    //bigquery.googleapis.com/projects/PROJECT_ID/datasets/DATASET_ID/tables/TABLE_ID
+            """
+
+            results_table: str = proto.Field(
+                proto.STRING,
+                number=1,
+            )
+
+        bigquery_export: "DataProfileSpec.PostScanActions.BigQueryExport" = proto.Field(
+            proto.MESSAGE,
+            number=1,
+            message="DataProfileSpec.PostScanActions.BigQueryExport",
+        )
+
+    class SelectedFields(proto.Message):
+        r"""The specification for fields to include or exclude in data
+        profile scan.
+
+        Attributes:
+            field_names (MutableSequence[str]):
+                Optional. Expected input is a list of fully
+                qualified names of fields as in the schema.
+
+                Only top-level field names for nested fields are
+                supported. For instance, if 'x' is of nested
+                field type, listing 'x' is supported but 'x.y.z'
+                is not supported. Here 'y' and 'y.z' are nested
+                fields of 'x'.
+        """
+
+        field_names: MutableSequence[str] = proto.RepeatedField(
+            proto.STRING,
+            number=1,
+        )
+
+    sampling_percent: float = proto.Field(
+        proto.FLOAT,
+        number=2,
+    )
+    row_filter: str = proto.Field(
+        proto.STRING,
+        number=3,
+    )
+    post_scan_actions: PostScanActions = proto.Field(
+        proto.MESSAGE,
+        number=4,
+        message=PostScanActions,
+    )
+    include_fields: SelectedFields = proto.Field(
+        proto.MESSAGE,
+        number=5,
+        message=SelectedFields,
+    )
+    exclude_fields: SelectedFields = proto.Field(
+        proto.MESSAGE,
+        number=6,
+        message=SelectedFields,
+    )
 
 
 class DataProfileResult(proto.Message):
-    r"""DataProfileResult defines the output of DataProfileScan.
-    Each field of the table will have field type specific profile
-    result.
+    r"""DataProfileResult defines the output of DataProfileScan. Each
+    field of the table will have field type specific profile result.
 
     Attributes:
         row_count (int):
-            The count of all rows in the sampled data.
-            Return 0, if zero rows.
+            The count of rows scanned.
         profile (google.cloud.dataplex_v1.types.DataProfileResult.Profile):
-            This represents the profile information per
-            field.
+            The profile information per field.
         scanned_data (google.cloud.dataplex_v1.types.ScannedData):
-            The data scanned for this profile.
+            The data scanned for this result.
+        post_scan_actions_result (google.cloud.dataplex_v1.types.DataProfileResult.PostScanActionsResult):
+            Output only. The result of post scan actions.
     """
 
     class Profile(proto.Message):
-        r"""Profile information describing the structure and layout of
-        the data and contains the profile info.
+        r"""Contains name, type, mode and field type specific profile
+        information.
 
         Attributes:
             fields (MutableSequence[google.cloud.dataplex_v1.types.DataProfileResult.Profile.Field]):
-                The sequence of fields describing data in
-                table entities.
+                List of fields with structural and profile
+                information for each field.
         """
 
         class Field(proto.Message):
-            r"""Represents a column field within a table schema.
+            r"""A field within a table.
 
             Attributes:
                 name (str):
                     The name of the field.
                 type_ (str):
-                    The field data type. Possible values include:
-
-                    -  STRING
-                    -  BYTE
-                    -  INT64
-                    -  INT32
-                    -  INT16
-                    -  DOUBLE
-                    -  FLOAT
-                    -  DECIMAL
-                    -  BOOLEAN
-                    -  BINARY
-                    -  TIMESTAMP
-                    -  DATE
-                    -  TIME
-                    -  NULL
-                    -  RECORD
+                    The data type retrieved from the schema of the data source.
+                    For instance, for a BigQuery native table, it is the
+                    `BigQuery Table
+                    Schema <https://cloud.google.com/bigquery/docs/reference/rest/v2/tables#tablefieldschema>`__.
+                    For a Dataplex Entity, it is the `Entity
+                    Schema <https://cloud.google.com/dataplex/docs/reference/rpc/google.cloud.dataplex.v1#type_3>`__.
                 mode (str):
-                    The mode of the field. Its value will be:
-                    REQUIRED, if it is a required field.
-                    NULLABLE, if it is an optional field.
-                    REPEATED, if it is a repeated field.
+                    The mode of the field. Possible values include:
+
+                    -  REQUIRED, if it is a required field.
+                    -  NULLABLE, if it is an optional field.
+                    -  REPEATED, if it is a repeated field.
                 profile (google.cloud.dataplex_v1.types.DataProfileResult.Profile.Field.ProfileInfo):
-                    The profile information for the corresponding
+                    Profile information for the corresponding
                     field.
             """
 
             class ProfileInfo(proto.Message):
-                r"""ProfileInfo defines the profile information for each schema
-                field type.
+                r"""The profile information for each field type.
 
                 This message has `oneof`_ fields (mutually exclusive fields).
                 For each oneof, at most one member field can be set at the same time.
@@ -107,46 +204,47 @@ class DataProfileResult(proto.Message):
 
                 Attributes:
                     null_ratio (float):
-                        The ratio of null rows against the rows in
-                        the sampled data.
+                        Ratio of rows with null value against total
+                        scanned rows.
                     distinct_ratio (float):
-                        The ratio of rows that are distinct against
-                        the rows in the sampled data.
+                        Ratio of rows with distinct values against
+                        total scanned rows. Not available for complex
+                        non-groupable field type RECORD and fields with
+                        REPEATABLE mode.
                     top_n_values (MutableSequence[google.cloud.dataplex_v1.types.DataProfileResult.Profile.Field.ProfileInfo.TopNValue]):
-                        The array of top N values of the field in the
-                        sampled data. Currently N is set as 10 or equal
-                        to distinct values in the field, whichever is
-                        smaller. This will be optional for complex
-                        non-groupable data-types such as JSON, ARRAY,
-                        JSON, STRUCT.
+                        The list of top N non-null values, frequency
+                        and ratio with which they occur in the scanned
+                        data. N is 10 or equal to the number of distinct
+                        values in the field, whichever is smaller. Not
+                        available for complex non-groupable field type
+                        RECORD and fields with REPEATABLE mode.
                     string_profile (google.cloud.dataplex_v1.types.DataProfileResult.Profile.Field.ProfileInfo.StringFieldInfo):
-                        The corresponding string field profile.
+                        String type field information.
 
                         This field is a member of `oneof`_ ``field_info``.
                     integer_profile (google.cloud.dataplex_v1.types.DataProfileResult.Profile.Field.ProfileInfo.IntegerFieldInfo):
-                        The corresponding integer field profile.
+                        Integer type field information.
 
                         This field is a member of `oneof`_ ``field_info``.
                     double_profile (google.cloud.dataplex_v1.types.DataProfileResult.Profile.Field.ProfileInfo.DoubleFieldInfo):
-                        The corresponding double field profile.
+                        Double type field information.
 
                         This field is a member of `oneof`_ ``field_info``.
                 """
 
                 class StringFieldInfo(proto.Message):
-                    r"""StringFieldInfo defines output info for any string type
-                    field.
+                    r"""The profile information for a string type field.
 
                     Attributes:
                         min_length (int):
-                            The minimum length of the string field in the
-                            sampled data. Optional if zero non-null rows.
+                            Minimum length of non-null values in the
+                            scanned data.
                         max_length (int):
-                            The maximum length of a string field in the
-                            sampled data. Optional if zero non-null rows.
+                            Maximum length of non-null values in the
+                            scanned data.
                         average_length (float):
-                            The average length of a string field in the
-                            sampled data. Optional if zero non-null rows.
+                            Average length of non-null values in the
+                            scanned data.
                     """
 
                     min_length: int = proto.Field(
@@ -163,23 +261,20 @@ class DataProfileResult(proto.Message):
                     )
 
                 class IntegerFieldInfo(proto.Message):
-                    r"""IntegerFieldInfo defines output for any integer type field.
+                    r"""The profile information for an integer type field.
 
                     Attributes:
                         average (float):
-                            The average of non-null values of integer
-                            field in the sampled data. Return NaN, if the
-                            field has a NaN. Optional if zero non-null rows.
+                            Average of non-null values in the scanned
+                            data. NaN, if the field has a NaN.
                         standard_deviation (float):
-                            The standard deviation of non-null of integer
-                            field in the sampled data. Return NaN, if the
-                            field has a NaN. Optional if zero non-null rows.
+                            Standard deviation of non-null values in the
+                            scanned data. NaN, if the field has a NaN.
                         min_ (int):
-                            The minimum value of an integer field in the
-                            sampled data. Return NaN, if the field has a
-                            NaN. Optional if zero non-null rows.
+                            Minimum of non-null values in the scanned
+                            data. NaN, if the field has a NaN.
                         quartiles (MutableSequence[int]):
-                            A quartile divide the number of data points
+                            A quartile divides the number of data points
                             into four parts, or quarters, of more-or-less
                             equal size. Three main quartiles used are: The
                             first quartile (Q1) splits off the lowest 25% of
@@ -191,13 +286,13 @@ class DataProfileResult(proto.Message):
                             quartile (Q3) splits off the highest 25% of data
                             from the lowest 75%. It is known as the upper or
                             75th empirical quartile, as 75% of the data lies
-                            below this point. So, here the quartiles is
-                            provided as an ordered list of quartile values,
-                            occurring in order Q1, median, Q3.
+                            below this point. Here, the quartiles is
+                            provided as an ordered list of approximate
+                            quartile values for the scanned data, occurring
+                            in order Q1, median, Q3.
                         max_ (int):
-                            The maximum value of an integer field in the
-                            sampled data. Return NaN, if the field has a
-                            NaN. Optional if zero non-null rows.
+                            Maximum of non-null values in the scanned
+                            data. NaN, if the field has a NaN.
                     """
 
                     average: float = proto.Field(
@@ -222,23 +317,20 @@ class DataProfileResult(proto.Message):
                     )
 
                 class DoubleFieldInfo(proto.Message):
-                    r"""DoubleFieldInfo defines output for any double type field.
+                    r"""The profile information for a double type field.
 
                     Attributes:
                         average (float):
-                            The average of non-null values of double
-                            field in the sampled data. Return NaN, if the
-                            field has a NaN. Optional if zero non-null rows.
+                            Average of non-null values in the scanned
+                            data. NaN, if the field has a NaN.
                         standard_deviation (float):
-                            The standard deviation of non-null of double
-                            field in the sampled data. Return NaN, if the
-                            field has a NaN. Optional if zero non-null rows.
+                            Standard deviation of non-null values in the
+                            scanned data. NaN, if the field has a NaN.
                         min_ (float):
-                            The minimum value of a double field in the
-                            sampled data. Return NaN, if the field has a
-                            NaN. Optional if zero non-null rows.
+                            Minimum of non-null values in the scanned
+                            data. NaN, if the field has a NaN.
                         quartiles (MutableSequence[float]):
-                            A quartile divide the numebr of data points
+                            A quartile divides the number of data points
                             into four parts, or quarters, of more-or-less
                             equal size. Three main quartiles used are: The
                             first quartile (Q1) splits off the lowest 25% of
@@ -250,13 +342,13 @@ class DataProfileResult(proto.Message):
                             quartile (Q3) splits off the highest 25% of data
                             from the lowest 75%. It is known as the upper or
                             75th empirical quartile, as 75% of the data lies
-                            below this point. So, here the quartiles is
-                            provided as an ordered list of quartile values,
-                            occurring in order Q1, median, Q3.
+                            below this point. Here, the quartiles is
+                            provided as an ordered list of quartile values
+                            for the scanned data, occurring in order Q1,
+                            median, Q3.
                         max_ (float):
-                            The maximum value of a double field in the
-                            sampled data. Return NaN, if the field has a
-                            NaN. Optional if zero non-null rows.
+                            Maximum of non-null values in the scanned
+                            data. NaN, if the field has a NaN.
                     """
 
                     average: float = proto.Field(
@@ -281,16 +373,18 @@ class DataProfileResult(proto.Message):
                     )
 
                 class TopNValue(proto.Message):
-                    r"""The TopNValue defines the structure of output of top N values
-                    of a field.
+                    r"""Top N non-null values in the scanned data.
 
                     Attributes:
                         value (str):
-                            The value is the string value of the actual
-                            value from the field.
+                            String value of a top N non-null value.
                         count (int):
-                            The frequency count of the corresponding
-                            value in the field.
+                            Count of the corresponding value in the
+                            scanned data.
+                        ratio (float):
+                            Ratio of the corresponding value in the field
+                            against the total number of rows in the scanned
+                            data.
                     """
 
                     value: str = proto.Field(
@@ -301,6 +395,10 @@ class DataProfileResult(proto.Message):
                         proto.INT64,
                         number=2,
                     )
+                    ratio: float = proto.Field(
+                        proto.DOUBLE,
+                        number=3,
+                    )
 
                 null_ratio: float = proto.Field(
                     proto.DOUBLE,
@@ -362,6 +460,64 @@ class DataProfileResult(proto.Message):
             message="DataProfileResult.Profile.Field",
         )
 
+    class PostScanActionsResult(proto.Message):
+        r"""The result of post scan actions of DataProfileScan job.
+
+        Attributes:
+            bigquery_export_result (google.cloud.dataplex_v1.types.DataProfileResult.PostScanActionsResult.BigQueryExportResult):
+                Output only. The result of BigQuery export
+                post scan action.
+        """
+
+        class BigQueryExportResult(proto.Message):
+            r"""The result of BigQuery export post scan action.
+
+            Attributes:
+                state (google.cloud.dataplex_v1.types.DataProfileResult.PostScanActionsResult.BigQueryExportResult.State):
+                    Output only. Execution state for the BigQuery
+                    exporting.
+                message (str):
+                    Output only. Additional information about the
+                    BigQuery exporting.
+            """
+
+            class State(proto.Enum):
+                r"""Execution state for the exporting.
+
+                Values:
+                    STATE_UNSPECIFIED (0):
+                        The exporting state is unspecified.
+                    SUCCEEDED (1):
+                        The exporting completed successfully.
+                    FAILED (2):
+                        The exporting is no longer running due to an
+                        error.
+                    SKIPPED (3):
+                        The exporting is skipped due to no valid scan
+                        result to export (usually caused by scan
+                        failed).
+                """
+                STATE_UNSPECIFIED = 0
+                SUCCEEDED = 1
+                FAILED = 2
+                SKIPPED = 3
+
+            state: "DataProfileResult.PostScanActionsResult.BigQueryExportResult.State" = proto.Field(
+                proto.ENUM,
+                number=1,
+                enum="DataProfileResult.PostScanActionsResult.BigQueryExportResult.State",
+            )
+            message: str = proto.Field(
+                proto.STRING,
+                number=2,
+            )
+
+        bigquery_export_result: "DataProfileResult.PostScanActionsResult.BigQueryExportResult" = proto.Field(
+            proto.MESSAGE,
+            number=1,
+            message="DataProfileResult.PostScanActionsResult.BigQueryExportResult",
+        )
+
     row_count: int = proto.Field(
         proto.INT64,
         number=3,
@@ -376,6 +532,11 @@ class DataProfileResult(proto.Message):
         number=5,
         message=processing.ScannedData,
     )
+    post_scan_actions_result: PostScanActionsResult = proto.Field(
+        proto.MESSAGE,
+        number=6,
+        message=PostScanActionsResult,
+    )
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
