@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2022 Google LLC
+# Copyright 2024 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,6 +43,8 @@ __protobuf__ = proto.module(
         "GetDataScanJobRequest",
         "ListDataScanJobsRequest",
         "ListDataScanJobsResponse",
+        "GenerateDataQualityRulesRequest",
+        "GenerateDataQualityRulesResponse",
         "DataScan",
         "DataScanJob",
     },
@@ -84,6 +86,9 @@ class CreateDataScanRequest(proto.Message):
             -  Must end with a number or a letter.
             -  Must be between 1-63 characters.
             -  Must be unique within the customer project / location.
+        validate_only (bool):
+            Optional. Only validate the request, but do not perform
+            mutations. The default is ``false``.
     """
 
     parent: str = proto.Field(
@@ -99,6 +104,10 @@ class CreateDataScanRequest(proto.Message):
         proto.STRING,
         number=3,
     )
+    validate_only: bool = proto.Field(
+        proto.BOOL,
+        number=4,
+    )
 
 
 class UpdateDataScanRequest(proto.Message):
@@ -111,6 +120,9 @@ class UpdateDataScanRequest(proto.Message):
             Only fields specified in ``update_mask`` are updated.
         update_mask (google.protobuf.field_mask_pb2.FieldMask):
             Required. Mask of fields to update.
+        validate_only (bool):
+            Optional. Only validate the request, but do not perform
+            mutations. The default is ``false``.
     """
 
     data_scan: "DataScan" = proto.Field(
@@ -123,6 +135,10 @@ class UpdateDataScanRequest(proto.Message):
         number=2,
         message=field_mask_pb2.FieldMask,
     )
+    validate_only: bool = proto.Field(
+        proto.BOOL,
+        number=3,
+    )
 
 
 class DeleteDataScanRequest(proto.Message):
@@ -194,7 +210,7 @@ class ListDataScansRequest(proto.Message):
         page_size (int):
             Optional. Maximum number of dataScans to
             return. The service may return fewer than this
-            value. If unspecified, at most 10 scans will be
+            value. If unspecified, at most 500 scans will be
             returned. The maximum value is 1000; values
             above 1000 will be coerced to 1000.
         page_token (str):
@@ -306,7 +322,7 @@ class GetDataScanJobRequest(proto.Message):
     Attributes:
         name (str):
             Required. The resource name of the DataScanJob:
-            ``projects/{project}/locations/{location_id}/dataScans/{data_scan_id}/dataScanJobs/{data_scan_job_id}``
+            ``projects/{project}/locations/{location_id}/dataScans/{data_scan_id}/jobs/{data_scan_job_id}``
             where ``project`` refers to a *project_id* or
             *project_number* and ``location_id`` refers to a GCP region.
         view (google.cloud.dataplex_v1.types.GetDataScanJobRequest.DataScanJobView):
@@ -361,6 +377,26 @@ class ListDataScanJobsRequest(proto.Message):
             subsequent page. When paginating, all other parameters
             provided to ``ListDataScanJobs`` must match the call that
             provided the page token.
+        filter (str):
+            Optional. An expression for filtering the results of the
+            ListDataScanJobs request.
+
+            If unspecified, all datascan jobs will be returned. Multiple
+            filters can be applied (with ``AND``, ``OR`` logical
+            operators). Filters are case-sensitive.
+
+            Allowed fields are:
+
+            -  ``start_time``
+            -  ``end_time``
+
+            ``start_time`` and ``end_time`` expect RFC-3339 formatted
+            strings (e.g. 2018-10-08T18:30:00-07:00).
+
+            For instance, 'start_time > 2018-10-08T00:00:00.123456789Z
+            AND end_time < 2018-10-09T00:00:00.123456789Z' limits
+            results to DataScanJobs between specified start and end
+            times.
     """
 
     parent: str = proto.Field(
@@ -375,6 +411,10 @@ class ListDataScanJobsRequest(proto.Message):
         proto.STRING,
         number=3,
     )
+    filter: str = proto.Field(
+        proto.STRING,
+        number=4,
+    )
 
 
 class ListDataScanJobsResponse(proto.Message):
@@ -404,6 +444,41 @@ class ListDataScanJobsResponse(proto.Message):
     )
 
 
+class GenerateDataQualityRulesRequest(proto.Message):
+    r"""Generate recommended DataQualityRules request.
+
+    Attributes:
+        name (str):
+            Required. The name should be either
+
+            -  the name of a datascan with at least one successful
+               completed data profiling job, or
+            -  the name of a successful completed data profiling
+               datascan job.
+    """
+
+    name: str = proto.Field(
+        proto.STRING,
+        number=1,
+    )
+
+
+class GenerateDataQualityRulesResponse(proto.Message):
+    r"""Generate recommended DataQualityRules response.
+
+    Attributes:
+        rule (MutableSequence[google.cloud.dataplex_v1.types.DataQualityRule]):
+            Generated recommended {@link
+            DataQualityRule}s.
+    """
+
+    rule: MutableSequence[data_quality.DataQualityRule] = proto.RepeatedField(
+        proto.MESSAGE,
+        number=1,
+        message=data_quality.DataQualityRule,
+    )
+
+
 class DataScan(proto.Message):
     r"""Represents a user-visible job which provides the insights for the
     related data source.
@@ -457,6 +532,7 @@ class DataScan(proto.Message):
             Required. The data source for DataScan.
         execution_spec (google.cloud.dataplex_v1.types.DataScan.ExecutionSpec):
             Optional. DataScan execution settings.
+
             If not specified, the fields in it will use
             their default values.
         execution_status (google.cloud.dataplex_v1.types.DataScan.ExecutionStatus):
